import logging
from typing import Any
import requests
from .enum import *


class Action:
    """
    go-cqhttp 实现的API
    """

    def __init__(self, http: str):
        """
        初始化
        :param http: 请求地址
        """
        self.__http = http
        self.__request = requests.Session()
        pass

    def do_post(self, uri: str, payload: dict[str, Any]) -> dict[str, Any] | None:
        """
        一个通用的post请求
        :param uri: 终结点
        :param payload: 参数
        :return: dict[str, Any] | None
        """
        headers = {'content-type': 'application/json; charset=utf-8'}
        api_url = f'{self.__http}{uri}'
        with self.__request.post(api_url, json=payload, headers=headers) as resp:
            if resp.status_code != 200:
                logging.error(resp.text)
                return None
            return resp.json()

    def send_private_msg(self, user_id: int, message: Any, group_id: int | None = None, auto_escape: bool = False) -> \
            dict[str, Any] | None:
        """
        发送私聊消息

        文档: https://docs.go-cqhttp.org/api/#%E5%8F%91%E9%80%81%E7%A7%81%E8%81%8A%E6%B6%88%E6%81%AF
        :param user_id: 对方 QQ 号
        :param message: 要发送的内容
        :param group_id: 主动发起临时会话时的来源群号(可选, 机器人本身必须是管理员/群主)
        :param auto_escape: 消息内容是否作为纯文本发送 ( 即不解析 CQ 码 ) , 只在 message 字段是字符串时有效
        :return: dict[str, Any] | None
        """
        uri = '/send_private_msg'
        payload = {
            'user_id': user_id,
            'message': message,
            'group_id': group_id,
            'auto_escape': auto_escape,
        }
        return self.do_post(uri, payload)

    def send_group_msg(self, group_id: int, message: Any, auto_escape: bool = False) -> dict[str, Any] | None:
        """
        发送群消息

        文档: https://docs.go-cqhttp.org/api/#%E5%8F%91%E9%80%81%E7%BE%A4%E6%B6%88%E6%81%AF
        :param group_id: 群号
        :param message: 要发送的内容
        :param auto_escape: 消息内容是否作为纯文本发送 ( 即不解析 CQ 码 ) , 只在 message 字段是字符串时有效
        :return: dict[str, Any] | None
        """
        uri = '/send_group_msg'
        payload = {
            'group_id': group_id,
            'message': message,
            'auto_escape': auto_escape,
        }
        return self.do_post(uri, payload)

    def send_group_forward_msg(self, group_id: int, messages: list[Any]) -> dict[str, Any] | None:
        """
        发送合并转发 ( 群 )

        文档: https://docs.go-cqhttp.org/api/#%E5%8F%91%E9%80%81%E5%90%88%E5%B9%B6%E8%BD%AC%E5%8F%91-%E7%BE%A4
        :param group_id: 群号
        :param messages: 自定义转发消息, 具体看 https://docs.go-cqhttp.org/cqcode/#%E5%90%88%E5%B9%B6%E8%BD%AC%E5%8F%91%E6%B6%88%E6%81%AF%E8%8A%82%E7%82%B9
        :return: dict[str, Any] | None
        """
        uri = '/send_group_forward_msg'
        payload = {
            'group_id': group_id,
            'messages': messages,
        }
        return self.do_post(uri, payload)

    def send_msg(self, message_type: PostMessageType, message: Any, user_id: int = 0, group_id: int = 0,
                 auto_escape: bool = False) -> dict[str, Any] | None:
        """
        发送群消息

        文档: https://docs.go-cqhttp.org/api/#%E5%8F%91%E9%80%81%E7%BE%A4%E6%B6%88%E6%81%AF
        :param message_type: 支持 private、group , 分别对应私聊、群组
        :param user_id: 对方 QQ 号 ( 消息类型为 private 时需要 )
        :param group_id: 群号 ( 消息类型为 group 时需要 )
        :param message: 要发送的内容
        :param auto_escape: 消息内容是否作为纯文本发送 ( 即不解析 CQ 码 ) , 只在 message 字段是字符串时有效
        :return: dict[str, Any] | None
        """
        uri = '/send_msg'
        payload = {
            'message_type': message_type,
            'user_id': user_id,
            'group_id': group_id,
            'message': message,
            'auto_escape': auto_escape,
        }
        return self.do_post(uri, payload)

    def delete_msg(self, message_id: int) -> dict[str, Any] | None:
        """
        撤回消息

        文档: https://docs.go-cqhttp.org/api/#%E6%92%A4%E5%9B%9E%E6%B6%88%E6%81%AF
        :param message_id: 消息ID
        :return: dict[str, Any] | None
        """
        uri = '/delete_msg'
        payload = {
            'message_id': message_id,
        }
        return self.do_post(uri, payload)

    def get_msg(self, message_id: int) -> dict[str, Any] | None:
        """
        获取消息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E6%B6%88%E6%81%AF
        :param message_id: 消息ID
        :return: dict[str, Any] | None
        """
        uri = '/get_msg'
        payload = {
            'message_id': message_id,
        }
        return self.do_post(uri, payload)

    def get_forward_msg(self, message_id: int) -> dict[str, Any] | None:
        """
        获取合并转发内容

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E5%90%88%E5%B9%B6%E8%BD%AC%E5%8F%91%E5%86%85%E5%AE%B9
        :param message_id: 对应合并转发中的 id 字段
        :return: dict[str, Any] | None
        """
        uri = '/get_forward_msg'
        payload = {
            'message_id': message_id,
        }
        return self.do_post(uri, payload)

    def get_image(self, file: str) -> dict[str, Any] | None:
        """
        获取图片信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E5%9B%BE%E7%89%87%E4%BF%A1%E6%81%AF
        :param file: 图片缓存文件名
        :return: dict[str, Any] | None
        """
        uri = '/get_image'
        payload = {
            'file': file,
        }
        return self.do_post(uri, payload)

    def mark_msg_as_read(self, message_id: int) -> dict[str, Any] | None:
        """
        标记消息已读

        文档: https://docs.go-cqhttp.org/api/#%E6%A0%87%E8%AE%B0%E6%B6%88%E6%81%AF%E5%B7%B2%E8%AF%BB
        :param message_id: 消息ID
        :return: dict[str, Any] | None
        """
        uri = '/mark_msg_as_read'
        payload = {
            'message_id': message_id,
        }
        return self.do_post(uri, payload)

    def set_group_kick(self, group_id: int, user_id: int, reject_add_request: bool = False) -> dict[str, Any] | None:
        """
        群组踢人

        文档: https://docs.go-cqhttp.org/api/#%E7%BE%A4%E7%BB%84%E8%B8%A2%E4%BA%BA
        :param group_id: 群号
        :param user_id: 要踢的 QQ 号
        :param reject_add_request: 拒绝此人的加群请求
        :return: dict[str, Any] | None
        """
        uri = '/set_group_kick'
        payload = {
            'group_id': group_id,
            'user_id': user_id,
            'reject_add_request': reject_add_request,
        }
        return self.do_post(uri, payload)

    def set_group_ban(self, group_id: int, user_id: int, duration: int = 1800) -> dict[str, Any] | None:
        """
        群组单人禁言

        文档: https://docs.go-cqhttp.org/api/#%E7%BE%A4%E7%BB%84%E5%8D%95%E4%BA%BA%E7%A6%81%E8%A8%80
        :param group_id: 群号
        :param user_id: 要踢的 QQ 号
        :param duration: 禁言时长, 单位秒, 0 表示取消禁言
        :return:
        """
        uri = '/set_group_ban'
        payload = {
            'group_id': group_id,
            'user_id': user_id,
            'duration': duration,
        }
        return self.do_post(uri, payload)

    def set_group_anonymous_ban(self, group_id: int, anonymous: Any = None, anonymous_flag: str = '',
                                duration: int = 1800) -> dict[str, Any] | None:
        """
        群组匿名用户禁言
        anonymous 和 anonymous_flag 两者任选其一传入即可, 若都传入, 则使用 anonymous。

        文档: https://docs.go-cqhttp.org/api/#%E7%BE%A4%E7%BB%84%E5%8C%BF%E5%90%8D%E7%94%A8%E6%88%B7%E7%A6%81%E8%A8%80
        :param group_id: 群号
        :param anonymous: 可选, 要禁言的匿名用户对象（群消息上报的 anonymous 字段）
        :param anonymous_flag: 可选, 要禁言的匿名用户的 flag（需从群消息上报的数据中获得）
        :param duration: 禁言时长, 单位秒, 0 表示取消禁言
        :return: dict[str, Any] | None
        """
        uri = '/set_group_anonymous_ban'
        payload = {
            'group_id': group_id,
            'anonymous': anonymous,
            'anonymous_flag': anonymous_flag,
            'duration': duration,
        }
        return self.do_post(uri, payload)

    def set_group_whole_ban(self, group_id: int, enable: bool = True) -> dict[str, Any] | None:
        """
        群组全员禁言

        文档: https://docs.go-cqhttp.org/api/#%E7%BE%A4%E7%BB%84%E5%85%A8%E5%91%98%E7%A6%81%E8%A8%80
        :param group_id: 群号
        :param enable: 是否禁言
        :return: dict[str, Any] | None
        """
        uri = '/set_group_whole_ban'
        payload = {
            'group_id': group_id,
            'enable': enable,
        }
        return self.do_post(uri, payload)

    def set_group_admin(self, group_id: int, user_id: int, enable: bool = True) -> dict[str, Any] | None:
        """
        群组设置管理员

        文档: https://docs.go-cqhttp.org/api/#%E7%BE%A4%E7%BB%84%E8%AE%BE%E7%BD%AE%E7%AE%A1%E7%90%86%E5%91%98
        :param group_id: 群号
        :param user_id: 要设置管理员的 QQ 号
        :param enable: true 为设置, false 为取消
        :return: dict[str, Any] | None
        """
        uri = '/set_group_admin'
        payload = {
            'group_id': group_id,
            'user_id': user_id,
            'enable': enable,
        }
        return self.do_post(uri, payload)

    def set_group_anonymous(self, group_id: int, enable: bool = True) -> dict[str, Any] | None:
        """
        群组匿名

        文档: https://docs.go-cqhttp.org/api/#%E7%BE%A4%E7%BB%84%E5%8C%BF%E5%90%8D
        :param group_id: 群号
        :param enable: 是否允许匿名聊天
        :return: dict[str, Any] | None
        """
        uri = '/set_group_anonymous'
        payload = {
            'group_id': group_id,
            'enable': enable,
        }
        return self.do_post(uri, payload)

    def set_group_card(self, group_id: int, user_id: int, card: str = '') -> dict[str, Any] | None:
        """
        设置群名片 ( 群备注 )

        文档: https://docs.go-cqhttp.org/api/#%E8%AE%BE%E7%BD%AE%E7%BE%A4%E5%90%8D%E7%89%87-%E7%BE%A4%E5%A4%87%E6%B3%A8
        :param group_id: 群号
        :param user_id: 要设置的 QQ 号
        :param card: 群名片内容, 不填或空字符串表示删除群名片
        :return: dict[str, Any] | None
        """
        uri = '/set_group_card'
        payload = {
            'group_id': group_id,
            'user_id': user_id,
            'card': card,
        }
        return self.do_post(uri, payload)

    def set_group_name(self, group_id: int, group_name: str) -> dict[str, Any] | None:
        """
        设置群名

        文档: https://docs.go-cqhttp.org/api/#%E8%AE%BE%E7%BD%AE%E7%BE%A4%E5%90%8D
        :param group_id: 群号
        :param group_name: 新群名
        :return: dict[str, Any] | None
        """
        uri = '/set_group_name'
        payload = {
            'group_id': group_id,
            'group_name': group_name,
        }
        return self.do_post(uri, payload)

    def set_group_leave(self, group_id: int, is_dismiss: bool = False) -> dict[str, Any] | None:
        """
        退出群组

        文档: https://docs.go-cqhttp.org/api/#%E9%80%80%E5%87%BA%E7%BE%A4%E7%BB%84
        :param group_id: 群号
        :param is_dismiss: 是否解散, 如果登录号是群主, 则仅在此项为 true 时能够解散
        :return: dict[str, Any] | None
        """
        uri = '/set_group_leave'
        payload = {
            'group_id': group_id,
            'is_dismiss': is_dismiss,
        }
        return self.do_post(uri, payload)

    def set_group_special_title(self, group_id: int, user_id: int, special_title: str = '', duration: int = -1) -> dict[
                                                                                                                       str, Any] | None:
        """
        设置群组专属头衔

        文档: https://docs.go-cqhttp.org/api/#%E8%AE%BE%E7%BD%AE%E7%BE%A4%E7%BB%84%E4%B8%93%E5%B1%9E%E5%A4%B4%E8%A1%94
        :param group_id: 群号
        :param user_id: 要设置的 QQ 号
        :param special_title: 专属头衔, 不填或空字符串表示删除专属头衔
        :param duration: 专属头衔有效期, 单位秒, -1 表示永久, 不过此项似乎没有效果, 可能是只有某些特殊的时间长度有效, 有待测试
        :return: dict[str, Any] | None
        """
        uri = '/set_group_special_title'
        payload = {
            'group_id': group_id,
            'user_id': user_id,
            'special_title': special_title,
            'duration': duration,
        }
        return self.do_post(uri, payload)

    def send_group_sign(self, group_id: int) -> dict[str, Any] | None:
        """
        群打卡

        文档: https://docs.go-cqhttp.org/api/#%E7%BE%A4%E6%89%93%E5%8D%A1
        :param group_id: 群号
        :return: dict[str, Any] | None
        """
        uri = '/send_group_sign'
        payload = {
            'group_id': group_id,
        }
        return self.do_post(uri, payload)

    def set_friend_add_request(self, flag: str, approve: bool = True, remark: str = '') -> dict[str, Any] | None:
        """
        处理加好友请求

        文档: https://docs.go-cqhttp.org/api/#%E5%A4%84%E7%90%86%E5%8A%A0%E5%A5%BD%E5%8F%8B%E8%AF%B7%E6%B1%82
        :param flag: 加好友请求的 flag（需从上报的数据中获得）
        :param approve: 是否同意请求
        :param remark: 添加后的好友备注（仅在同意时有效）
        :return: dict[str, Any] | None
        """
        uri = '/set_friend_add_request'
        payload = {
            'flag': flag,
            'approve': approve,
            'remark': remark,
        }
        return self.do_post(uri, payload)

    def set_group_add_request(self, flag: str, sub_type: str, approve: bool = True, reason: str = '') -> dict[
                                                                                                             str, Any] | None:
        """
        处理加群请求／邀请

        文档: https://docs.go-cqhttp.org/api/#%E5%A4%84%E7%90%86%E5%8A%A0%E7%BE%A4%E8%AF%B7%E6%B1%82-%E9%82%80%E8%AF%B7
        :param flag: 加群请求的 flag（需从上报的数据中获得）
        :param sub_type: add 或 invite, 请求类型（需要和上报消息中的 sub_type 字段相符）
        :param approve: 是否同意请求／邀请
        :param reason: 拒绝理由（仅在拒绝时有效）
        :return: dict[str, Any] | None
        """
        uri = '/set_group_add_request'
        payload = {
            'flag': flag,
            'sub_type': sub_type,
            'approve': approve,
            'reason': reason,
        }
        return self.do_post(uri, payload)

    def get_login_info(self) -> dict[str, Any] | None:
        """
        获取登录号信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%99%BB%E5%BD%95%E5%8F%B7%E4%BF%A1%E6%81%AF
        :return: dict[str, Any] | None
        """
        uri = '/get_login_info'
        payload = {}
        return self.do_post(uri, payload)

    def set_qq_profile(self, nickname: str, company: str, email: str, college: str, personal_note: str) -> dict[
                                                                                                               str, Any] | None:
        """
        设置登录号资料

        文档: https://docs.go-cqhttp.org/api/#%E8%AE%BE%E7%BD%AE%E7%99%BB%E5%BD%95%E5%8F%B7%E8%B5%84%E6%96%99
        :param nickname: 名称
        :param company: 公司
        :param email: 邮箱
        :param college: 学校
        :param personal_note: 个人说明
        :return: dict[str, Any] | None
        """
        uri = '/set_qq_profile'
        payload = {
            'nickname': nickname,
            'company': company,
            'email': email,
            'college': college,
            'personal_note': personal_note,
        }
        return self.do_post(uri, payload)

    def get_stranger_info(self, user_id: int, no_cache: bool = False) -> dict[str, Any] | None:
        """
        获取陌生人信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E9%99%8C%E7%94%9F%E4%BA%BA%E4%BF%A1%E6%81%AF
        :param user_id: QQ 号
        :param no_cache: 是否不使用缓存（使用缓存可能更新不及时, 但响应更快）
        :return: dict[str, Any] | None
        """
        uri = '/get_stranger_info'
        payload = {
            'user_id': user_id,
            'no_cache': no_cache,
        }
        return self.do_post(uri, payload)

    def get_friend_list(self) -> dict[str, Any] | None:
        """
        获取好友列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E5%A5%BD%E5%8F%8B%E5%88%97%E8%A1%A8
        :return: dict[str, Any] | None
        """
        uri = '/get_friend_list'
        payload = {}
        return self.do_post(uri, payload)

    def get_unidirectional_friend_list(self) -> dict[str, Any] | None:
        """
        获取单向好友列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E5%8D%95%E5%90%91%E5%A5%BD%E5%8F%8B%E5%88%97%E8%A1%A8
        :return: dict[str, Any] | None
        """
        uri = '/get_unidirectional_friend_list'
        payload = {}
        return self.do_post(uri, payload)

    def delete_friend(self, user_id: int) -> dict[str, Any] | None:
        """
        删除好友

        文档: https://docs.go-cqhttp.org/api/#%E5%88%A0%E9%99%A4%E5%A5%BD%E5%8F%8B
        :param user_id: 好友 QQ 号
        :return: dict[str, Any] | None
        """
        uri = '/delete_friend'
        payload = {
            'user_id': user_id,
        }
        return self.do_post(uri, payload)

    def get_group_info(self, group_id: int, no_cache: bool = False) -> dict[str, Any] | None:
        """
        获取群信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E4%BF%A1%E6%81%AF
        :param group_id: 群号
        :param no_cache: 是否不使用缓存（使用缓存可能更新不及时, 但响应更快）
        :return: dict[str, Any] | None
        """
        uri = '/get_group_info'
        payload = {
            'group_id': group_id,
            'no_cache': no_cache,
        }
        return self.do_post(uri, payload)

    def get_group_list(self, no_cache: bool = False) -> dict[str, Any] | None:
        """
        获取群列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E5%88%97%E8%A1%A8
        :param no_cache: 是否不使用缓存（使用缓存可能更新不及时, 但响应更快）
        :return: dict[str, Any] | None
        """
        uri = '/get_group_list'
        payload = {
            'no_cache': no_cache
        }
        return self.do_post(uri, payload)

    def get_group_member_info(self, group_id: int, user_id: int, no_cache: bool = False) -> dict[str, Any] | None:
        """
        获取群成员信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E6%88%90%E5%91%98%E4%BF%A1%E6%81%AF
        :param group_id: 群号
        :param user_id: QQ 号
        :param no_cache: 是否不使用缓存（使用缓存可能更新不及时, 但响应更快）
        :return: dict[str, Any] | None
        """
        uri = '/get_group_member_info'
        payload = {
            'group_id': group_id,
            'user_id': user_id,
            'no_cache': no_cache,
        }
        return self.do_post(uri, payload)

    def get_group_member_list(self, group_id: int, no_cache: bool = False) -> dict[str, Any] | None:
        """
        获取群成员列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E6%88%90%E5%91%98%E5%88%97%E8%A1%A8
        :param group_id: 群号
        :param no_cache: 是否不使用缓存（使用缓存可能更新不及时, 但响应更快）
        :return: dict[str, Any] | None
        """
        uri = '/get_group_member_list'
        payload = {
            'group_id': group_id,
            'no_cache': no_cache,
        }
        return self.do_post(uri, payload)

    def get_group_honor_info(self, group_id: int, type: str) -> dict[str, Any] | None:
        """
        获取群荣誉信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E8%8D%A3%E8%AA%89%E4%BF%A1%E6%81%AF
        :param group_id: 群号
        :param type: 要获取的群荣誉类型, 可传入 talkative performer legend strong_newbie emotion 以分别获取单个类型的群荣誉数据, 或传入 all 获取所有数据
        :return: dict[str, Any] | None
        """
        uri = '/get_group_honor_info'
        payload = {
            'group_id': group_id,
            'type': type,
        }
        return self.do_post(uri, payload)

    def can_send_image(self) -> dict[str, Any] | None:
        """
        检查是否可以发送图片

        文档: https://docs.go-cqhttp.org/api/#%E6%A3%80%E6%9F%A5%E6%98%AF%E5%90%A6%E5%8F%AF%E4%BB%A5%E5%8F%91%E9%80%81%E5%9B%BE%E7%89%87
        :return: dict[str, Any] | None
        """
        uri = '/can_send_image'
        payload = {}
        return self.do_post(uri, payload)

    def can_send_record(self) -> dict[str, Any] | None:
        """
        检查是否可以发送语音

        文档: https://docs.go-cqhttp.org/api/#%E6%A3%80%E6%9F%A5%E6%98%AF%E5%90%A6%E5%8F%AF%E4%BB%A5%E5%8F%91%E9%80%81%E8%AF%AD%E9%9F%B3
        :return: dict[str, Any] | None
        """
        uri = '/can_send_record'
        payload = {}
        return self.do_post(uri, payload)

    def get_version_info(self, x: int) -> dict[str, Any] | None:
        """
        获取版本信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%89%88%E6%9C%AC%E4%BF%A1%E6%81%AF
        :param x:
        :return: dict[str, Any] | None
        """
        uri = '/get_version_info'
        payload = {
            'x': x,
        }
        return self.do_post(uri, payload)

    def set_group_portrait(self, group_id: int, file: str, cache: int) -> dict[str, Any] | None:
        """
        设置群头像

        文档: https://docs.go-cqhttp.org/api/#%E8%AE%BE%E7%BD%AE%E7%BE%A4%E5%A4%B4%E5%83%8F
        :param group_id: 群号
        :param file: 图片文件名
        :param cache: 表示是否使用已缓存的文件
        :return: dict[str, Any] | None
        """
        uri = '/set_group_portrait'
        payload = {
            'group_id': group_id,
            'file': file,
            'cache': cache,
        }
        return self.do_post(uri, payload)

    def get_group_system_msg(self, invited_requests: list[Any], join_requests: list[Any]) -> dict[str, Any] | None:
        """
        获取群系统消息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E7%B3%BB%E7%BB%9F%E6%B6%88%E6%81%AF
        :param invited_requests: 邀请消息列表
        :param join_requests: 进群消息列表
        :return: dict[str, Any] | None
        """
        uri = '/get_group_system_msg'
        payload = {
            'invited_requests': invited_requests,
            'join_requests': join_requests,
        }
        return self.do_post(uri, payload)

    def upload_private_file(self, group_id: int, file: str, name: str) -> dict[str, Any] | None:
        """
        上传私聊文件

        文档: https://docs.go-cqhttp.org/api/#%E4%B8%8A%E4%BC%A0%E7%A7%81%E8%81%8A%E6%96%87%E4%BB%B6
        :param group_id: 群号
        :param file: 本地文件路径
        :param name: 储存名称
        :return: dict[str, Any] | None
        """
        uri = '/upload_private_file'
        payload = {
            'group_id': group_id,
            'file': file,
            'name': name,
        }
        return self.do_post(uri, payload)

    def upload_group_file(self, group_id: int, file: str, name: str, folder: str) -> dict[str, Any] | None:
        """
        上传群文件

        文档: https://docs.go-cqhttp.org/api/#%E4%B8%8A%E4%BC%A0%E7%BE%A4%E6%96%87%E4%BB%B6
        :param group_id: 群号
        :param file: 本地文件路径
        :param name: 储存名称
        :param folder: 父目录ID
        :return: dict[str, Any] | None
        """
        uri = '/upload_group_file'
        payload = {
            'group_id': group_id,
            'file': file,
            'name': name,
            'folder': folder,
        }
        return self.do_post(uri, payload)

    def get_group_file_system_info(self, group_id: int) -> dict[str, Any] | None:
        """
        获取群文件系统信息

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E6%96%87%E4%BB%B6%E7%B3%BB%E7%BB%9F%E4%BF%A1%E6%81%AF
        :param group_id: 群号
        :return: dict[str, Any] | None
        """
        uri = '/get_group_file_system_info'
        payload = {
            'group_id': group_id,
        }
        return self.do_post(uri, payload)

    def get_group_root_files(self, group_id: int) -> dict[str, Any] | None:
        """
        获取群根目录文件列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E6%A0%B9%E7%9B%AE%E5%BD%95%E6%96%87%E4%BB%B6%E5%88%97%E8%A1%A8
        :param group_id: 群号
        :return: dict[str, Any] | None
        """
        uri = '/get_group_root_files'
        payload = {
            'group_id': group_id,
        }
        return self.do_post(uri, payload)

    def get_group_files_by_folder(self, group_id: int, folder_id: str) -> dict[str, Any] | None:
        """
        获取群子目录文件列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E5%AD%90%E7%9B%AE%E5%BD%95%E6%96%87%E4%BB%B6%E5%88%97%E8%A1%A8
        :param group_id: 群号
        :param folder_id: 文件夹ID 参考 Folder 对象
        :return: dict[str, Any] | None
        """
        uri = '/get_group_files_by_folder'
        payload = {
            'group_id': group_id,
            'folder_id': folder_id,
        }
        return self.do_post(uri, payload)

    def create_group_file_folder(self, group_id: int, name: str, parent_id: str = '/') -> dict[str, Any] | None:
        """
        创建群文件文件夹
        仅能在根目录创建文件夹

        文档: https://docs.go-cqhttp.org/api/#%E5%88%9B%E5%BB%BA%E7%BE%A4%E6%96%87%E4%BB%B6%E6%96%87%E4%BB%B6%E5%A4%B9
        :param group_id: 群号
        :param name: 文件夹名称
        :param parent_id: 仅能为 /
        :return: dict[str, Any] | None
        """
        uri = '/create_group_file_folder'
        payload = {
            'group_id': group_id,
            'name': name,
            'parent_id': parent_id,
        }
        return self.do_post(uri, payload)

    def delete_group_folder(self, group_id: int, folder_id: str) -> dict[str, Any] | None:
        """
        删除群文件文件夹

        文档: https://docs.go-cqhttp.org/api/#%E5%88%A0%E9%99%A4%E7%BE%A4%E6%96%87%E4%BB%B6%E6%96%87%E4%BB%B6%E5%A4%B9
        :param group_id: 群号
        :param folder_id: 文件夹ID 参考 Folder 对象
        :return: dict[str, Any] | None
        """
        uri = '/delete_group_folder'
        payload = {
            'group_id': group_id,
            'folder_id': folder_id,
        }
        return self.do_post(uri, payload)

    def delete_group_file(self, group_id: int, file_id: str, busid: int) -> dict[str, Any] | None:
        """
        删除群文件

        文档: https://docs.go-cqhttp.org/api/#%E5%88%A0%E9%99%A4%E7%BE%A4%E6%96%87%E4%BB%B6
        :param group_id: 群号
        :param file_id: 文件ID 参考 File 对象
        :param busid: 文件类型 参考 File 对象
        :return: dict[str, Any] | None
        """
        uri = '/delete_group_file'
        payload = {
            'group_id': group_id,
            'file_id': file_id,
            'busid': busid,
        }
        return self.do_post(uri, payload)

    def get_group_file_url(self, group_id: int, file_id: str, busid: int) -> dict[str, Any] | None:
        """
        获取群文件资源链接

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E6%96%87%E4%BB%B6%E8%B5%84%E6%BA%90%E9%93%BE%E6%8E%A5
        :param group_id: 群号
        :param file_id: 文件ID 参考 File 对象
        :param busid: 文件类型 参考 File 对象
        :return: dict[str, Any] | None
        """
        uri = '/get_group_file_url'
        payload = {
            'group_id': group_id,
            'file_id': file_id,
            'busid': busid,
        }
        return self.do_post(uri, payload)

    def get_status(self) -> dict[str, Any] | None:
        """
        获取状态

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%8A%B6%E6%80%81
        :return: dict[str, Any] | None
        """
        uri = '/get_status'
        payload = {}
        return self.do_post(uri, payload)

    def get_group_at_all_remain(self, group_id: int) -> dict[str, Any] | None:
        """
        获取群 @全体成员 剩余次数

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4-%E5%85%A8%E4%BD%93%E6%88%90%E5%91%98-%E5%89%A9%E4%BD%99%E6%AC%A1%E6%95%B0
        :param group_id: 群号
        :return: dict[str, Any] | None
        """
        uri = '/get_group_at_all_remain'
        payload = {
            'group_id': group_id,
        }
        return self.do_post(uri, payload)

    def _send_group_notice(self, group_id: int, content: str, image: str = '') -> dict[str, Any] | None:
        """
        发送群公告

        文档: https://docs.go-cqhttp.org/api/#%E5%8F%91%E9%80%81%E7%BE%A4%E5%85%AC%E5%91%8A
        :param group_id: 群号
        :param content: 公告内容
        :param image: 图片路径（可选）
        :return: dict[str, Any] | None
        """
        uri = '/_send_group_notice'
        payload = {
            'group_id': group_id,
            'content': content,
            'image': image,
        }
        return self.do_post(uri, payload)

    def _get_group_notice(self, group_id: int) -> dict[str, Any] | None:
        """
        获取群公告

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E5%85%AC%E5%91%8A
        :param group_id: 群号
        :return: dict[str, Any] | None
        """
        uri = '/_get_group_notice'
        payload = {
            'group_id': group_id,
        }
        return self.do_post(uri, payload)

    def reload_event_filter(self, file: str) -> dict[str, Any] | None:
        """
        重载事件过滤器

        文档: https://docs.go-cqhttp.org/api/#%E9%87%8D%E8%BD%BD%E4%BA%8B%E4%BB%B6%E8%BF%87%E6%BB%A4%E5%99%A8
        :param file: 事件过滤器文件
        :return: dict[str, Any] | None
        """
        uri = '/reload_event_filter'
        payload = {
            'file': file,
        }
        return self.do_post(uri, payload)

    def download_file(self, url: str, thread_count: int, headers: str | list) -> dict[str, Any] | None:
        """
        下载文件到缓存目录

        文档: https://docs.go-cqhttp.org/api/#%E4%B8%8B%E8%BD%BD%E6%96%87%E4%BB%B6%E5%88%B0%E7%BC%93%E5%AD%98%E7%9B%AE%E5%BD%95
        :param url: 链接地址
        :param thread_count: 下载线程数
        :param headers: 自定义请求头
        :return: dict[str, Any] | None
        """
        uri = '/download_file'
        payload = {
            'url': url,
            'thread_count': thread_count,
            'headers': headers,
        }
        return self.do_post(uri, payload)

    def get_online_clients(self, no_cache: bool) -> dict[str, Any] | None:
        """
        获取当前账号在线客户端列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E5%BD%93%E5%89%8D%E8%B4%A6%E5%8F%B7%E5%9C%A8%E7%BA%BF%E5%AE%A2%E6%88%B7%E7%AB%AF%E5%88%97%E8%A1%A8
        :param no_cache: 是否无视缓存
        :return: dict[str, Any] | None
        """
        uri = '/get_online_clients'
        payload = {
            'no_cache': no_cache,
        }
        return self.do_post(uri, payload)

    def get_group_msg_history(self, message_seq: int, group_id: int) -> dict[str, Any] | None:
        """
        获取群消息历史记录

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%BE%A4%E6%B6%88%E6%81%AF%E5%8E%86%E5%8F%B2%E8%AE%B0%E5%BD%95
        :param message_seq: 起始消息序号, 可通过 get_msg 获得
        :param group_id: 群号
        :return: dict[str, Any] | None
        """
        uri = '/get_group_msg_history'
        payload = {
            'message_seq': message_seq,
            'group_id': group_id,
        }
        return self.do_post(uri, payload)

    def set_essence_msg(self, message_id: int) -> dict[str, Any] | None:
        """
        设置精华消息

        文档: https://docs.go-cqhttp.org/api/#%E8%AE%BE%E7%BD%AE%E7%B2%BE%E5%8D%8E%E6%B6%88%E6%81%AF
        :param message_id: 消息ID
        :return: dict[str, Any] | None
        """
        uri = '/set_essence_msg'
        payload = {
            'message_id': message_id,
        }
        return self.do_post(uri, payload)

    def delete_essence_msg(self, message_id: int) -> dict[str, Any] | None:
        """
        移出精华消息

        文档: https://docs.go-cqhttp.org/api/#%E7%A7%BB%E5%87%BA%E7%B2%BE%E5%8D%8E%E6%B6%88%E6%81%AF
        :param message_id: 消息ID
        :return: dict[str, Any] | None
        """
        uri = '/delete_essence_msg'
        payload = {
            'message_id': message_id,
        }
        return self.do_post(uri, payload)

    def get_essence_msg_list(self, group_id: int) -> dict[str, Any] | None:
        """
        获取精华消息列表

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E7%B2%BE%E5%8D%8E%E6%B6%88%E6%81%AF%E5%88%97%E8%A1%A8
        :param group_id: 群号
        :return: dict[str, Any] | None
        """
        uri = '/get_essence_msg_list'
        payload = {
            'group_id': group_id,
        }
        return self.do_post(uri, payload)

    def check_url_safely(self, url: str) -> dict[str, Any] | None:
        """
        检查链接安全性

        文档: https://docs.go-cqhttp.org/api/#%E6%A3%80%E6%9F%A5%E9%93%BE%E6%8E%A5%E5%AE%89%E5%85%A8%E6%80%A7
        :param url: 需要检查的链接
        :return: dict[str, Any] | None
        """
        uri = '/check_url_safely'
        payload = {
            'url': url,
        }
        return self.do_post(uri, payload)

    def _get_model_show(self, model: str) -> dict[str, Any] | None:
        """
        获取在线机型

        文档: https://docs.go-cqhttp.org/api/#%E8%8E%B7%E5%8F%96%E5%9C%A8%E7%BA%BF%E6%9C%BA%E5%9E%8B
        :param model: 机型名称
        :return: dict[str, Any] | None
        """
        uri = '/_get_model_show'
        payload = {
            'model': model,
        }
        return self.do_post(uri, payload)

    def _set_model_show(self, model: str, model_show: str) -> dict[str, Any] | None:
        """
        设置在线机型

        文档: https://docs.go-cqhttp.org/api/#%E8%AE%BE%E7%BD%AE%E5%9C%A8%E7%BA%BF%E6%9C%BA%E5%9E%8B
        :param model: 机型名称
        :param model_show:
        :return: dict[str, Any] | None
        """
        uri = '/_set_model_show'
        payload = {
            'model': model,
            'model_show': model_show,
        }
        return self.do_post(uri, payload)

    def delete_unidirectional_friend(self, user_id: int) -> dict[str, Any] | None:
        """
        删除单向好友

        文档: https://docs.go-cqhttp.org/api/#%E5%88%A0%E9%99%A4%E5%8D%95%E5%90%91%E5%A5%BD%E5%8F%8B
        :param user_id: 单向好友QQ号
        :return: dict[str, Any] | None
        """
        uri = '/delete_unidirectional_friend'
        payload = {
            'user_id': user_id,
        }
        return self.do_post(uri, payload)

    def send_private_forward_msg(self, user_id: int, messages: list[Any]) -> dict[str, Any] | None:
        """
        发送合并转发 ( 好友 )

        文档: https://docs.go-cqhttp.org/api/#%E5%8F%91%E9%80%81%E5%90%88%E5%B9%B6%E8%BD%AC%E5%8F%91-%E5%A5%BD%E5%8F%8B
        :param user_id: 好友QQ号
        :param messages: 自定义转发消息, 具体看 https://docs.go-cqhttp.org/cqcode/#%E5%90%88%E5%B9%B6%E8%BD%AC%E5%8F%91%E6%B6%88%E6%81%AF%E8%8A%82%E7%82%B9
        :return: dict[str, Any] | None
        """
        uri = '/send_private_forward_msg'
        payload = {
            'user_id': user_id,
            'messages': messages,
        }
        return self.do_post(uri, payload)


__all__ = ['Action']


from enum import Enum, unique


@unique
class PostType(str, Enum):
    """
    枚举: 上报类型

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-type
    """
    # 消息
    MESSAGE = 'message'
    # 消息发送
    MESSAGE_SENT = 'message_sent'
    # 请求
    REQUEST = 'request'
    # 通知
    NOTICE = 'notice'
    # 元事件
    META_EVENT = 'meta_event'


@unique
class PostMessageType(str, Enum):
    """
    枚举: 消息类型

    文档：https://docs.go-cqhttp.org/reference/data_struct.html#post-message-type
    """
    # 私聊
    PRIVATE = 'private'
    # 群聊
    GROUP = 'group'

    def __str__(self) -> str:
        return self.value


@unique
class PostMessageSubType(str, Enum):
    """
    枚举: 消息子类型

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-message-type
    """
    # 好友
    FRIEND = 'friend'
    # 群聊
    NORMAL = 'normal'
    # 匿名
    ANONYMOUS = 'anonymous'
    # 群中自身发送
    GROUP_SELF = 'group_self'
    # 群临时会话
    GROUP = 'group'
    # 系统提示
    NOTICE = 'notice'

    def __str__(self) -> str:
        return self.value


@unique
class PostMessageTempSource(int, Enum):
    """
    枚举：消息来源
    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-message-tempsource
    """
    # 群聊
    GROUP = 0
    # qq咨询
    QQ_ZIXUN = 1
    # 查找
    LOOKUP = 2
    # qq电影
    QQ_FILM = 3
    # 热聊
    HOT_CHAT = 4
    # 验证消息
    VERIFY_MESSAGE = 6
    # 多人聊天
    MULTI_PERSON_CHAT = 7
    # 约会
    APPOINTMENT = 8
    # 通讯录
    ADDRESS_LIST = 9

    def __str__(self) -> int:
        return self.value


@unique
class PostRequestType(str, Enum):
    """
    枚举: 请求类型

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-request-type
    """
    # 好友
    FRIEND = 'friend'
    # 群
    GROUP = 'group'

    def __str__(self) -> str:
        return self.value


@unique
class PostNoticeType(str, Enum):
    """
    枚举: 通知类型

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-notice-type
    """
    # 群文件上传
    GROUP_UPLOAD = 'group_upload'
    # 群管理员变更
    GROUP_ADMIN = 'group_admin'
    # 群成员减少
    GROUP_DECREASE = 'group_decrease'
    # 群成员增加
    GROUP_INCREASE = 'group_increase'
    # 群成员禁言
    GROUP_BAN = 'group_ban'
    # 好友添加
    FRIEND_ADD = 'friend_add'
    # 群消息撤回
    GROUP_RECALL = 'group_recall'
    # 好友消息撤回
    FRIEND_RECALL = 'friend_recall'
    # 群名片变更
    GROUP_CARD = 'group_card'
    # 离线文件上传
    OFFLINE_FILE = 'offline_file'
    # 客户端状态变更
    CLIENT_STATUS = 'client_status'
    # 精华消息
    ESSENCE = 'essence'
    # 系统通知
    NOTIFY = 'notify'

    def __str__(self) -> str:
        return self.value


@unique
class PostNoticeNotifySubType(str, Enum):
    """
    枚举: 系统通知的子类型

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-notice-notify-subtype
    """
    # 群荣誉变更
    HONOR = 'honor'
    # 戳一戳
    POKE = 'poke'
    # 群红包幸运王
    LUCKY_KING = 'lucky_king'
    # 群成员头衔变更
    TITLE = 'title'

    def __str__(self) -> str:
        return self.value


@unique
class PostMetaEventType(str, Enum):
    """
    枚举: 元事件类型

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-metaevent-type
    """
    # 生命周期
    LIFECYCLE = 'lifecycle'
    # 心跳包
    HEARTBEAT = 'heartbeat'

    def __str__(self) -> str:
        return self.value


@unique
class PostMetaEventLifecycleType(str, Enum):
    """
    枚举: 生命周期上报的子类型

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-metaevent-lifecycletype
    """
    # 启用
    ENABLE = 'enable'
    # 禁用
    DISABLE = 'disable'
    # 连接
    CONNECT = 'connect'

    def __str__(self) -> str:
        return self.value


__all__ = [
    'PostType',
    'PostMessageType',
    'PostMessageSubType',
    'PostMessageTempSource',
    'PostRequestType',
    'PostNoticeType',
    'PostNoticeNotifySubType',
    'PostMetaEventType',
    'PostMetaEventLifecycleType',
]



import re
from typing import Any
from .action import *
from .enum import *


class EventPost(object):
    """
    事件上报有效通用数据
    """
    # 事件发生的unix时间戳
    time: int
    # 收到事件的机器人的 QQ 号
    self_id: int
    # 表示该上报的类型, 消息, 消息发送, 请求, 通知, 或元事件
    post_type: PostType

    def __init__(self, message_data: dict[str, Any]):
        self.time = int(message_data['time'])
        self.self_id = int(message_data['self_id'])
        self.post_type = PostType(message_data['post_type'])


class PostMessageMessageSender:
    """
    消息发送者的信息

    文档: https://docs.go-cqhttp.org/reference/data_struct.html#post-message-messagesender
    """

    # 发送者 QQ 号
    user_id: int
    # 昵称
    nickname: str
    # 性别, male 或 female 或 unknown
    sex: str
    # 年龄
    age: int

    # ------- 当私聊类型为群临时会话时的额外字段 -------
    group_id: int | None

    # ------- 群聊时的额外字段 -------
    # 群名片／备注
    card: str | None
    # 地区
    area: str | None
    # 成员等级
    level: str | None
    # 角色, owner 或 admin 或 member
    role: str | None
    # 专属头衔
    title: str | None

    def __init__(self, sender: dict[str, Any]):
        self.user_id = int(sender['user_id'])
        self.age = int(sender['age'])
        self.nickname = sender['nickname']
        self.sex = sender['sex']

        self.group_id = sender.get('group_id', None)

        self.card = sender.get('card', None)
        self.area = sender.get('area', None)
        self.level = sender.get('level', None)
        self.role = sender.get('role', None)
        self.title = sender.get('title', None)


class Anonymous:
    """
    匿名信息
    """
    # 匿名用户id
    id: int
    # 匿名用户昵称
    name: str
    # 匿名用户 flag, 在调用禁言 API 时需要传入
    flag: str

    def __init__(self, anonymous: dict[str, Any]):
        self.id = anonymous.get('id', 0)
        self.name = anonymous.get('name', '')
        self.flag = anonymous.get('flag', '')


office_bots = [
    # q群管家
    2854196310,
    # 热议小助手
    2854209495,
    # 小冰
    2854196306,
    # 占卜师喵吉
    2854196318,
    # 暗区突围老皮
    2854203945,
    # 小YOYO
    2854203763,
    # 清安
    2854200554,
    # 小世界情报菌
    2854200454,
    # 玄中记卷卷
    2854214686,
    # 卧龙吟2乔妹
    2854211316,
    # 赞噢机器人
    2854204259,
    # QQ官方
    2854196320,
    # QQ小店助手
    2854196925,
    # Eva-创造恋人
    2854205672,
    # 小微欢乐多
    2854196324,
    # 表情包老铁
    2854196312,
    # 王者荣耀小狐狸
    2854196311,
    # 看点直播小助理
    2854215747,
    # DNF小酱油
    2854214473,
    # 英雄联盟 撸妹儿
    2854209507,
    # CODM-瘦普
    2854214941,
    # 火影忍者-豚豚
    2854211389,
    # 梦想新大陆-鲁鲁
    2854198999,
    # 黎明觉醒秀研
    2854216101,
    # 和平精英-小几
    2854196316,
    # 蒸汽机器人
    2854213468,
    # 奇迹Angela
    2854206897,
    # 小森村百科
    2854214963,
    # 饭团团
    2854211892,
    # 小莹机器人
    2854213893,
    # 阿暖
    2854202380,
    # 腾讯灯塔
    2854200812,
    # 蓝鲸信息流
    2854200449,
]


class EventMessage(EventPost):
    """
    post_type 为 message 或 message_sent 的上报将会有以下有效通用数据

    文档: https://docs.go-cqhttp.org/event/#%E6%B6%88%E6%81%AF%E4%B8%8A%E6%8A%A5
    """

    # 消息类型
    message_type: PostMessageType
    # 表示消息的子类型
    sub_type: PostMessageSubType
    # 消息 ID
    message_id: int
    # 发送者 QQ 号
    user_id: int
    # 一个消息链
    message: Any
    # CQ 码格式的消息
    raw_message: str
    # 字体
    font: int
    # 发送者信息
    sender: PostMessageMessageSender
    # 下面属性非通用数据
    # 消息序号
    message_seq: int | None
    # 临时对话的来源
    temp_source: PostMessageTempSource | None
    # 群ID
    group_id: str | None
    # 匿名信息, 如果不是匿名消息则为 null
    anonymous: Anonymous | None

    def __init__(self, message_data: dict[str, Any]):
        super().__init__(message_data)
        self.message_type = PostMessageType(message_data['message_type'])
        self.sub_type = PostMessageSubType(message_data['sub_type'])
        self.message_id = int(message_data['message_id'])
        self.user_id = int(message_data['user_id'])
        self.font = int(message_data['font'])
        self.raw_message = message_data['raw_message']
        self.message = message_data['message']
        self.sender = PostMessageMessageSender(message_data['sender'])

        self.message_seq = message_data.get('message_seq', None)
        if self.message_type == PostMessageType.PRIVATE:
            temp_source = message_data.get('temp_source', None)
            if temp_source is not None:
                self.temp_source = PostMessageTempSource(temp_source)
        elif self.message_type == PostMessageType.GROUP:
            self.group_id = message_data.get('group_id', None)
            anonymous = message_data.get('anonymous', None)
            if anonymous is not None:
                self.anonymous = Anonymous(anonymous)

    def is_at(self, qq: int | str = 0) -> bool:
        """
        某人是否被提及，默认是当前机器人
        :param qq: 查看指定QQ号是否被提及，或则all查看是不是提及了所有人，默认是查看当前机器人qq是否被提及
        :return:
        """
        if qq == 0:
            qq = self.self_id

        rule = rf'(\[CQ:at,qq\={qq}(,[a-z]+\=\S+)*\])'
        result = re.findall(rule, self.message)
        if len(result) == 0:
            return False
        return True

    def is_office_bot(self, qq: int = 0) -> bool:
        """
        当前消息是否是官方机器人发送的，也可以指定QQ查看是否是官方机器人
        :return:
        """
        if qq == 0:
            qq = self.user_id
        return qq in office_bots


class EventRequest(EventPost):
    """
    post_type 为 request 的上报会有以下有效通用数据

    文档: https://docs.go-cqhttp.org/event/#%E8%AF%B7%E6%B1%82%E4%B8%8A%E6%8A%A5
    """
    # 请求类型
    request_type: PostRequestType
    # 下面属性非通用数据
    # 发送请求的 QQ 号
    user_id: int | None
    # 验证信息
    comment: str | None
    # 请求 flag, 在调用处理请求的 API 时需要传入
    flag: str | None
    # add、invite 请求子类型, 分别表示加群请求、邀请登录号入群
    sub_type: str | None
    # 群号
    group_id: str | None

    def __init__(self, message_data: dict[str, Any]):
        super().__init__(message_data)
        self.request_type = PostRequestType(message_data['request_type'])

        self.user_id = message_data.get('user_id', None)
        self.comment = message_data.get('comment', None)
        self.flag = message_data.get('flag', None)
        self.sub_type = message_data.get('sub_type', None)
        self.group_id = message_data.get('group_id', None)


class EventNotice(EventPost):
    """
    post_type 为 notice 的上报会有以下有效通用数据

    文档: https://docs.go-cqhttp.org/event/#%E9%80%9A%E7%9F%A5%E4%B8%8A%E6%8A%A5
    """
    # 请求类型
    notice_type: PostNoticeType
    # 下面属性非通用数据
    # 私聊消息撤回[notice_type = friend_recall] 好友QQ
    # 群消息撤回[notice_type = group_recall] 消息发送者 QQ 号
    # 群成员增加[notice_type = group_increase] 加入者 QQ 号
    # 群成员减少[notice_type = group_decrease] 离开者 QQ 号
    # 群管理员变动[notice_type = group_decrease] 管理员 QQ 号
    # 群文件上传[notice_type = group_upload] 发送者 QQ 号
    # 群禁言[notice_type = group_ban] 被禁言 QQ 号 (为全员禁言时为0)
    # 好友添加[notice_type = friend_add] 新添加好友 QQ 号
    # 戳一戳（双击头像）[notice_type = notify, sub_type = poke] 发送者 QQ 号
    # 群红包运气王提示[notice_type = notify, sub_type = lucky_king] 红包发送者
    # 群成员荣誉变更提示[notice_type = notify, sub_type = honor] 成员id
    # 群成员头衔变更[notice_type = notify, sub_type = title] 变更头衔的用户 QQ 号
    # 群成员名片更新[notice_type = group_card] 成员id
    # 接收到离线文件[notice_type = offline_file] 发送者id
    user_id: int | None
    # 被撤回的消息 ID
    message_id: int | None
    # 群号
    group_id: int | None
    # 操作者 QQ 号
    operator_id: int | None
    # 群成员增加[notice_type = group_increase]事件子类型, 分别表示管理员已同意入群、管理员邀请入群: approve、invite
    # 群成员减少[notice_type = group_decrease]事件子类型, 分别表示主动退群、成员被踢、登录号被踢: leave、kick、kick_me
    # 群管理员变动[notice_type = group_admin]事件子类型, 分别表示设置和取消管理员: set、unset
    # 群禁言[notice_type = group_ban]事件子类型, 分别表示禁言、解除禁言: ban、lift_ban
    # 精华消息变更[notice_type = essence]事件子类型, 分别表示添加、移除: add、delete
    # 戳一戳（双击头像）[notice_type = notify, sub_type = poke]
    # 群红包运气王提示[notice_type = notify, sub_type = lucky_king]
    # 群成员荣誉变更提示[notice_type = notify, sub_type = honor]
    # 群成员头衔变更[notice_type = notify, sub_type = title]
    sub_type: str | None
    # 禁言时长, 单位秒 (为全员禁言时为-1)
    duration: int | None
    # 文件信息
    # 文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%96%87%E4%BB%B6%E4%B8%8A%E4%BC%A0
    file: dict[str, Any] | None
    # 发送者 QQ 号
    sender_id: int | None
    # 戳一戳（双击头像）[notice_type = notify, sub_type = poke] 被戳者
    # 群红包运气王提示[notice_type = notify, sub_type = lucky_king] 运气王id
    target_id: int | None
    # 荣誉类型: talkative - 龙王 、performer - 群聊之火 、emotion - 快乐源泉
    honor_type: str | None
    # 获得的新头衔
    title: str | None
    # 群成员名片更新 - 新名片
    card_new: str | None
    # 群成员名片更新 - 旧名片
    card_old: str | None
    # 客户端信息
    client: Any | None
    # 当前是否在线
    online: bool | None

    def __init__(self, message_data: dict[str, Any]):
        super().__init__(message_data)
        self.notice_type = PostNoticeType(message_data['notice_type'])

        self.user_id = message_data.get('user_id', None)
        self.message_id = message_data.get('message_id', None)
        self.group_id = message_data.get('group_id', None)
        self.operator_id = message_data.get('operator_id', None)
        self.sub_type = message_data.get('sub_type', None)
        self.duration = message_data.get('duration', None)
        self.file = message_data.get('file', None)
        self.sender_id = message_data.get('sender_id', None)
        self.target_id = message_data.get('target_id', None)
        self.honor_type = message_data.get('honor_type', None)
        self.title = message_data.get('title', None)
        self.card_new = message_data.get('card_new', None)
        self.card_old = message_data.get('card_old', None)
        self.client = message_data.get('client', None)
        self.online = message_data.get('online', None)


class EventMetaEvent(EventPost):
    """
    post_type 为 meta_event 的上报会有以下有效数据

    文档: https://docs.go-cqhttp.org/event/#%E5%85%83%E4%BA%8B%E4%BB%B6%E4%B8%8A%E6%8A%A5
    """
    # 请求类型
    meta_event_type: PostMetaEventType
    # 下面属性非通用数据
    # 应用程序状态
    status: dict[str, Any] | None
    # 距离上一次心跳包的时间(单位是毫秒)
    interval: int | None
    # 生命周期子类型: enable, disable, connect
    sub_type: str | None

    def __init__(self, message_data: dict[str, Any]):
        super().__init__(message_data)
        self.meta_event_type = PostMetaEventType(message_data['meta_event_type'])
        self.status = message_data.get('status', None)
        self.interval = message_data.get('interval', None)
        self.sub_type = message_data.get('sub_type', None)


# 事件
class Event:

    # -------- post - message ----------

    def on_message_private(self, act: Action, msg: EventMessage):
        """
        私聊消息

        文档: https://docs.go-cqhttp.org/event/#%E7%A7%81%E8%81%8A%E6%B6%88%E6%81%AF
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_message_group(self, act: Action, msg: EventMessage):
        """
        群消息

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%B6%88%E6%81%AF
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    # -------- post - notice ----------

    def on_notice_friend_recall(self, act: Action, msg: EventNotice):
        """
        私聊消息撤回

        文档: https://docs.go-cqhttp.org/event/#%E7%A7%81%E8%81%8A%E6%B6%88%E6%81%AF%E6%92%A4%E5%9B%9E
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_group_recall(self, act: Action, msg: EventNotice):
        """
        群消息撤回

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%B6%88%E6%81%AF%E6%92%A4%E5%9B%9E
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_group_increase(self, act: Action, msg: EventNotice):
        """
        群成员增加

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%88%90%E5%91%98%E5%A2%9E%E5%8A%A0
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_group_decrease(self, act: Action, msg: EventNotice):
        """
        群成员减少

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%88%90%E5%91%98%E5%87%8F%E5%B0%91
        :param msg:
        :return:
        """
        pass

    def on_notice_group_admin(self, act: Action, msg: EventNotice):
        """
        群管理员变动

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E7%AE%A1%E7%90%86%E5%91%98%E5%8F%98%E5%8A%A8
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_group_upload(self, act: Action, msg: EventNotice):
        """
        群文件上传

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%96%87%E4%BB%B6%E4%B8%8A%E4%BC%A0
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_group_ban(self, act: Action, msg: EventNotice):
        """
        群禁言

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E7%A6%81%E8%A8%80
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_friend_add(self, act: Action, msg: EventNotice):
        """
        好友添加

        文档: https://docs.go-cqhttp.org/event/#%E5%A5%BD%E5%8F%8B%E6%B7%BB%E5%8A%A0
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_notify_poke(self, act: Action, msg: EventNotice):
        """
        戳一戳（双击头像）
        包含好友、群内

        文档: https://docs.go-cqhttp.org/event/#%E5%A5%BD%E5%8F%8B%E6%88%B3%E4%B8%80%E6%88%B3-%E5%8F%8C%E5%87%BB%E5%A4%B4%E5%83%8F
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_notify_lucky_king(self, act: Action, msg: EventNotice):
        """
        群红包运气王提示

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E7%BA%A2%E5%8C%85%E8%BF%90%E6%B0%94%E7%8E%8B%E6%8F%90%E7%A4%BA
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_notify_honor(self, act: Action, msg: EventNotice):
        """
        群成员荣誉变更提示

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%88%90%E5%91%98%E8%8D%A3%E8%AA%89%E5%8F%98%E6%9B%B4%E6%8F%90%E7%A4%BA
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_notify_title(self, act: Action, msg: EventNotice):
        """
        群成员头衔变更

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%88%90%E5%91%98%E5%A4%B4%E8%A1%94%E5%8F%98%E6%9B%B4
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_group_card(self, act: Action, msg: EventNotice):
        """
        群成员名片更新

        文档: https://docs.go-cqhttp.org/event/#%E7%BE%A4%E6%88%90%E5%91%98%E5%90%8D%E7%89%87%E6%9B%B4%E6%96%B0
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_offline_file(self, act: Action, msg: EventNotice):
        """
        接收到离线文件

        文档: https://docs.go-cqhttp.org/event/#%E6%8E%A5%E6%94%B6%E5%88%B0%E7%A6%BB%E7%BA%BF%E6%96%87%E4%BB%B6
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_client_status(self, act: Action, msg: EventNotice):
        """
        其他客户端在线状态变更

        文档: https://docs.go-cqhttp.org/event/#%E5%85%B6%E4%BB%96%E5%AE%A2%E6%88%B7%E7%AB%AF%E5%9C%A8%E7%BA%BF%E7%8A%B6%E6%80%81%E5%8F%98%E6%9B%B4
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_notice_essence(self, act: Action, msg: EventNotice):
        """
        精华消息变更

        文档: https://docs.go-cqhttp.org/event/#%E7%B2%BE%E5%8D%8E%E6%B6%88%E6%81%AF%E5%8F%98%E6%9B%B4
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    # -------- post - request ----------

    def on_request_friend(self, act: Action, msg: EventRequest):
        """
        加好友请求

        文档: https://docs.go-cqhttp.org/event/#%E5%8A%A0%E5%A5%BD%E5%8F%8B%E8%AF%B7%E6%B1%82
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_request_group(self, msg: EventRequest):
        """
        加群请求／邀请

        文档: https://docs.go-cqhttp.org/event/#%E5%8A%A0%E7%BE%A4%E8%AF%B7%E6%B1%82-%E9%82%80%E8%AF%B7
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    # -------- post - meta ----------

    def on_meta_event_heartbeat(self, act: Action, msg: EventMetaEvent):
        """
        心跳包

        文档: https://docs.go-cqhttp.org/event/#%E5%BF%83%E8%B7%B3%E5%8C%85
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass

    def on_meta_event_lifecycle(self, act: Action, msg: EventMetaEvent):
        """
        生命周期

        文档: https://docs.go-cqhttp.org/event/#%E7%94%9F%E5%91%BD%E5%91%A8%E6%9C%9F
        :param act: api方法
        :param msg: 消息体
        :return:
        """
        pass


__all__ = [
    'EventPost',
    'PostMessageMessageSender',
    'Anonymous',
    'EventMessage',
    'EventRequest',
    'EventNotice',
    'EventMetaEvent',
    'Event',
]


from typing import List, Callable, Any


def escape(s, quote=True):
    """
    转义
    :param s:
    :param quote:
    :return:
    """
    s = s.replace("&", "&amp;")  # Must be done first!
    s = s.replace("<", "&lt;")
    s = s.replace(">", "&gt;")
    s = s.replace("[", "&#91;")
    s = s.replace("]", "&#93;")
    s = s.replace(",", "&#44;")
    if quote:
        s = s.replace('"', "&quot;")
        s = s.replace('\'', "&#x27;")
    return s


def g(key: str, value: Any, cond: Callable[[Any], bool], escape: bool = False, fail_break: bool = False):
    """
    :param key:
    :param value:
    :param cond: lambda 条件
    :param escape: 是否需要转义
    :param fail_break: 条件Fales是否直接return
    :return:
    """
    return [key, value, cond, escape, fail_break]


def attr_join(attrs: list[List]) -> str:
    """
    属性拼接

    示例: [
            [
                'key',
                value,
                lambda表达式,
                是否转义,
                是否表达式结果为False,直接返回
            ]
    ]
    :param attrs:
    :return:
    """
    items = []
    for v in attrs:
        if v[2](v[1]):
            val = v[1]
            if v[3]:
                val = escape(val)
            items.append(f'{v[0]}={val}')
            continue
        if v[4]:
            return ''
    return ','.join(items)


class CQ:
    """
    CQ码
    仅实现了可发送且没有风险的CQ码

    文档: https://docs.go-cqhttp.org/cqcode
    """

    @classmethod
    def face(cls, id: int) -> str:
        """
        qq 表情

        返回例子:  [CQ:face,id=123]

        文档: https://docs.go-cqhttp.org/cqcode/#qq-%E8%A1%A8%E6%83%85
        :param id: 表情id
        :return: str
        """
        if id < 0:
            return ''

        return f'[CQ:face,id={id}]'

    @classmethod
    def record(cls,
               file: str = '',
               magic: int = 0,
               url: str = '',
               cache: int = 1,
               proxy: int = 1,
               timeout: int = 0) -> str:
        """
        语音

        返回例子:  [CQ:record,file=http://baidu.com/1.mp3]

        文档: https://docs.go-cqhttp.org/cqcode/#%E8%AF%AD%E9%9F%B3
        :param file: 语音文件名
        :param magic: 发送时可选, 默认 0, 设置为 1 表示变声
        :param url: 语音 URL
        :param cache: 只在通过网络 URL 发送时有效, 表示是否使用已缓存的文件, 默认 1
        :param proxy: 只在通过网络 URL 发送时有效, 表示是否通过代理下载文件 ( 需通过环境变量或配置文件配置代理 ) , 默认 1
        :param timeout: 只在通过网络 URL 发送时有效, 单位秒, 表示下载网络文件的超时时间 , 默认不超时
        :return: str
        """

        file = file.strip()

        attrs = attr_join([
            g('file', file, lambda x: x != '', True, True),
            g('magic', magic, lambda x: x == 1),
        ])

        if attrs != '':
            return f'[CQ:record,{attrs}]'

        attrs = attr_join([
            g('url', url, lambda x: x != '', True, True),
            g('cache', cache, lambda x: x != 1),
            g('proxy', proxy, lambda x: x != 1),
            g('timeout', timeout, lambda x: x > 1),
        ])

        if attrs != '':
            return f'[CQ:record,{attrs}]'

        return ''

    @classmethod
    def video(cls, file: str, cover: str = '', c: int = 0) -> str:
        """
        短视频

        返回例子:  [CQ:video,file=http://baidu.com/1.mp4]

        文档: https://docs.go-cqhttp.org/cqcode/#%E7%9F%AD%E8%A7%86%E9%A2%91
        :param file: 视频地址, 支持http和file发送
        :param cover: 视频封面, 支持http, file和base64发送, 格式必须为jpg
        :param c: 通过网络下载视频时的线程数, 默认单线程. (在资源不支持并发时会自动处理)
        :return: str
        """
        attrs = attr_join([
            g('file', file, lambda x: x != '', True, True),
            g('cover', cover, lambda x: x != '', True),
            g('c', c, lambda x: x > 0),
        ])

        if attrs == '':
            return ''

        return f'[CQ:video,{attrs}]'

    @classmethod
    def at(cls, qq: int | str = 'all', name: str = '') -> str:
        """
        @某人

        返回例子:  [CQ:at,qq=123,name=不在群的QQ]

        文档: https://docs.go-cqhttp.org/cqcode/#qq-%E8%A1%A8%E6%83%85
        :param qq: @的 QQ 号, all 表示全体成员
        :param name: 当在群中找不到此QQ号的名称时才会生效
        :return: str
        """

        attrs = attr_join([
            g('qq', qq, lambda x: True),
            g('name', name, lambda x: x != '', True),
        ])

        return f'[CQ:at,{attrs}]'

    @classmethod
    def share(cls, url: str, title: str, content: str = '', image: str = '') -> str:
        """
        链接分享

        返回例子:  [CQ:share,url=http://baidu.com,title=百度]

        文档: https://docs.go-cqhttp.org/cqcode/#%E9%93%BE%E6%8E%A5%E5%88%86%E4%BA%AB
        :param url: URL
        :param title: 标题
        :param content: 发送时可选, 内容描述
        :param image: 发送时可选, 图片 URL
        :return: str
        """

        attrs = attr_join([
            g('url', url, lambda x: x != '', True, True),
            g('title', title, lambda x: x != '', True, True),
            g('content', content, lambda x: x != '', True),
            g('image', image, lambda x: x != '', True),
        ])

        if attrs == '':
            return ''

        return f'[CQ:share,{attrs}]'

    @classmethod
    def music(cls, type: str, id: int = 0, url: str = '', audio: str = '', title: str = '', content: str = '',
              image: str = '') -> str:
        """
        音乐分享

        返回例子:  [CQ:music,type=163,id=28949129]

        文档: https://docs.go-cqhttp.org/cqcode/#%E9%9F%B3%E4%B9%90%E5%88%86%E4%BA%AB
        :param type: qq 163 xm custom, 分别表示使用 QQ 音乐、网易云音乐、虾米音乐、自定义
        :param id: 歌曲 ID
        :param url: 当type=custom时生效: 点击后跳转目标 URL
        :param audio: 当type=custom时生效: 音乐 URL
        :param title: 当type=custom时生效: 标题
        :param content: 当type=custom时生效: 发送时可选, 内容描述
        :param image: 当type=custom时生效: 发送时可选, 图片 URL
        :return: str
        """

        attrs = attr_join([
            g('type', type, lambda x: x != '', False, True),
            g('id', id, lambda x: x > 0),
            g('url', url, lambda x: x != '', True),
            g('audio', audio, lambda x: x != '', True),
            g('title', title, lambda x: x != '', True),
            g('content', content, lambda x: x != '', True),
            g('image', image, lambda x: x != '', True),
        ])

        if attrs == '':
            return ''

        return f'[CQ:music,{attrs}]'

    @classmethod
    def image(cls, file: str, type: str = '', subType: int = 0, url: str = '', cache: int = 1, id: int = 40000,
              c: int = 1) -> str:
        """
        图片

        返回例子:  [CQ:image,file=http://baidu.com/1.jpg,type=show,id=40004]

        文档: https://docs.go-cqhttp.org/cqcode/#%E5%9B%BE%E7%89%87
        :param file: 图片文件名，支持：绝对路径、网络 URL、Base64 编码
        :param type: 图片类型, flash 表示闪照, show 表示秀图, 默认普通图片
        :param subType: 图片子类型, 只出现在群聊.
        :param url: 图片 URL
        :param cache: 只在通过网络 URL 发送时有效, 表示是否使用已缓存的文件, 默认 1
        :param id: 发送秀图时的特效id, 默认为40000
        :param c: 通过网络下载图片时的线程数, 默认单线程. (在资源不支持并发时会自动处理)
        :return: str
        """

        attrs = attr_join([
            g('file', file, lambda x: x != '', True, True),
            g('type', type, lambda x: x != ''),
            g('subType', subType, lambda x: x != 0),
            g('url', url, lambda x: x != '', True),
            g('cache', cache, lambda x: x != 1),
            g('id', id, lambda x: x != 40000),
            g('c', c, lambda x: x > 1),
        ])

        if attrs == '':
            return ''

        return f'[CQ:image,{attrs}]'

    @classmethod
    def reply(cls, id: int, text: str = '', qq: int = 0, time: int = 0, seq: int = 0) -> str:
        """
        回复

        如果 id 和 text 同时存在, 将采用自定义reply并替换原有信息 如果 id 获取失败, 将回退到自定义reply

        返回例子:  [CQ:reply,text=Hello World,qq=10086,time=3376656000,seq=5123]

        文档: https://docs.go-cqhttp.org/cqcode/#%E5%9B%9E%E5%A4%8D
        :param id: 回复时所引用的消息id, 必须为本群消息.
        :param text: 自定义回复的信息
        :param qq: 自定义回复时的自定义QQ, 如果使用自定义信息必须指定.
        :param time: 自定义回复时的时间, 格式为Unix时间
        :param seq: 起始消息序号, 可通过 get_msg 获得
        :return: str
        """
        attrs = attr_join([
            g('id', id, lambda x: x > 0, False, True),
            g('text', text, lambda x: x != '', True),
            g('qq', qq, lambda x: x > 0),
            g('time', time, lambda x: x > 0),
            g('seq', seq, lambda x: x > 0),
        ])

        if attrs == '':
            return ''

        return f'[CQ:reply,{attrs}]'

    @classmethod
    def poke(cls, qq: int) -> str:
        """
        戳一戳

        发送戳一戳消息无法撤回, 返回的 message id 恒定为 0

        返回例子:  [CQ:poke,qq=123456]

        文档: https://docs.go-cqhttp.org/cqcode/#%E6%88%B3%E4%B8%80%E6%88%B3
        :param qq: 需要戳的成员
        :return: str
        """
        if qq <= 0:
            return ''

        return f'[CQ:poke,qq={qq}]'

    @classmethod
    def gift(cls, qq: int, id: int) -> str:
        """
        礼物

        仅支持免费礼物, 发送群礼物消息 无法撤回, 返回的 message id 恒定为 0

        返回例子:  [CQ:gift,qq=123456,id=8]

        文档: https://docs.go-cqhttp.org/cqcode/#%E7%A4%BC%E7%89%A9
        :param qq: 需要戳的成员
        :param id: 礼物的类型
        :return: str
        """
        attrs = attr_join([
            g('qq', qq, lambda x: x > 0, False, True),
            g('id', id, lambda x: x > 0, False, True),
        ])

        if attrs == '':
            return ''

        return f'[CQ:gift,{attrs}]'

    @classmethod
    def tts(cls, text: str) -> str:
        """
        文本转语音

        通过TX的TTS接口, 采用的音源与登录账号的性别有关

        返回例子:  [CQ:tts,text=这是一条测试消息]

        文档: https://docs.go-cqhttp.org/cqcode/#%E6%96%87%E6%9C%AC%E8%BD%AC%E8%AF%AD%E9%9F%B3
        :param text: text
        :return: str
        """
        if text == '':
            return ''

        return f'[CQ:tts,text={text}]'


__all__ = ['CQ']



import re
from typing import Callable, List, Any
from .action import Action
from .event import EventMessage

# def handler(act: Action, msg: EventMessage, args: List[Any]) -> bool
CmdHandler = Callable[[Action, EventMessage, List[Any]], bool] | None

rule = r'\[CQ:[\S ]+?\]'


class Cmd:
    __cmds: dict[str, CmdHandler]
    __helps: dict[str, str]
    __commanders: dict[str, List[int]]

    def __init__(self, admins: List[int] = None):
        """
        初始化
        :param admins: 指定只有管理员才能使用所有的指令
        """
        self.__cmds = {}
        self.__helps = {}
        self.__commanders = {}
        self.admins = admins
        pass

    def add(self, cmd: str, help: str, handler: CmdHandler = None, commander: List[int] | None = None):
        """
        添加指令和处理方法
        :param cmd: 指令名字
        :param help: 指令说明文案
        :param handler: 处理方法，可以先空着不实现
        :param commander: 指定某个qq才能使用这个指令, 不指定则所有人可使用
        :return:
        """
        self.__cmds[cmd] = handler
        self.__helps[cmd] = help
        if commander is not None:
            self.__commanders[cmd] = commander
        pass

    def help(self) -> str:
        """
        返回所有指令的使用方法
        :return:
        """
        if len(self.__helps) == 0:
            return ''
        items = ['指令说明: ']
        for key in self.__helps:
            items.append(f'{key} - {self.__helps[key]}')
        return '\n'.join(items)

    def run(self, act: Action, msg: EventMessage) -> bool:
        """
        执行指令
        :param msg:
        :return:
        """
        # 机器人的消息过滤
        if msg.self_id == msg.user_id:
            return False
        if self.admins is not None and msg.user_id not in self.admins:
            return False
        # 将CQ码替换成空
        prompt = re.sub(rule, ' ', msg.message).strip()
        # 根据空格切割参数
        args = re.split(r'\s+', prompt)
        if len(args) == 0:
            return False
        handler = self.__cmds.get(args[0], None)
        if handler is None:
            return False
        commander = self.__commanders.get(args[0], None)
        if commander is not None and msg.user_id not in commander:
            return False
        return handler(act, msg, args[1:])


__all__ = ['Cmd', 'CmdHandler']


from setuptools import setup, find_packages

"""
打包和发布: 
python setup.py sdist bdist_wheel
pip install twine
twine upload dist/*
"""

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="cqbot",
    version="0.0.3",
    description="go-cqhttp python 框架，可以用于快速塔建 bot",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    license="MIT",
    url="https://github.com/lhlyu/cqbot",
    author="lhlyu",
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[
        "requests",
        "pyyaml",
        "websocket-client"
    ],
    python_requires='>=3.10'
)


import json
import logging
import os
import platform
import subprocess
import time
from typing import Callable, Any
import websocket
import yaml
from .event import *
from .enum import *
from .action import *

CONFIG_FILE = "./config.yml"
LOG_FORMAT = "[%(asctime)s]-[%(levelname)s]: %(message)s"
DATE_FORMAT = "%Y-%m-%d %H:%M:%S"


class Bot(Event):

    version: str = '0.0.3'

    def __init__(self, loglevel: int = logging.INFO, path: str = './'):
        """
        初始化
        :param loglevel: 日志等级，默认 logging.INFO
        :param path: config.yml 和 go-cqhttp 所在目录，这两个东西必须放在一起
        """
        super().__init__()
        # 检查config.yml是否合法
        self.__check = False
        # http请求的地址
        self.__http_address = None
        # websocket需要连接的地址
        self.__ws_address = None
        self.__path = path
        self.__cfg = os.path.join(path, CONFIG_FILE)
        # 所有api
        self.__action = None
        # 所有事件
        self.__events = {}
        logging.basicConfig(level=loglevel, format=LOG_FORMAT, datefmt=DATE_FORMAT)

    def run(self):
        """
        运行bot
        :return:
        """

        # 检查是否有config.yml文件
        if not os.path.exists(self.__cfg):
            raise FileExistsError('config.yml文件不存在')

        # 检查配置文件
        self.__check_config()

        if self.__check is False:
            raise Exception('config.yml配置错误，请仔细检查')

        # 运行go-cqhttp
        self.__run_go_cqhttp()

        # 初始化事件
        self.__init_events()

        # 初始化action, 所有的API操作都在这里
        self.__action = Action(self.__http_address)

        # 连接ws
        self.__run_ws()

    def __check_config(self):
        """
        检查配置文件
        :return:
        """
        with open(self.__cfg, "r") as f:
            yml = yaml.load(f.read(), Loader=yaml.Loader)
            servers = yml.get('servers', None)
            if servers is None:
                raise Exception('config.yml文件缺少属性servers')
            if not isinstance(servers, list):
                raise Exception('config.yml属性servers不是list类型')
            for server in servers:
                for k, v in server.items():
                    if k == 'http':
                        if not isinstance(v, dict):
                            raise Exception('config.yml属性servers.http.address不是dict类型')
                        self.__http_address = v.get('address', None)
                        if self.__http_address is None:
                            raise Exception('config.yml属性servers.http.address不能为空')

                        self.__http_address = f'http://{self.__http_address}'

                    if k == 'ws':
                        if not isinstance(v, dict):
                            raise Exception('config.yml属性servers.ws.address不是dict类型')
                        self.__ws_address = v.get('address', None)
                        if self.__ws_address is None:
                            raise Exception('config.yml属性servers.ws.address不能为空')

                        self.__ws_address = f'ws://{self.__ws_address}'

        self.__check = True

    def __run_go_cqhttp(self):
        """
        运行go_cqhttp
        :return:
        """
        plat = platform.system().lower()
        if plat == 'windows':
            subp = subprocess.Popen(f"cd {self.__path} && .\\go-cqhttp.exe -faststart", shell=True,
                                    stdout=subprocess.PIPE)
        else:
            subp = subprocess.Popen(f"cd {self.__path} && ./go-cqhttp -faststart", shell=True, stdout=subprocess.PIPE)

        while True:
            # 等待成功启动的信号
            shell_msg = subp.stdout.readline().decode("utf-8").strip()
            if shell_msg.strip() == "":
                continue

            print(shell_msg)
            if "CQ WebSocket 服务器已启动" in shell_msg:
                time.sleep(1)
                return

    def __run_ws(self):
        """
        运行websocket连接go-cqhttp
        :return:
        """

        def on_message(ws, msg):
            data = json.loads(msg)
            self.__call(data)

        def on_error(ws, error):
            logging.error(error)

        def on_close(ws):
            logging.info("websocket连接关闭")

        def on_open(ws):
            logging.info("cqbot当前版本: " + self.version)
            logging.info("websocket连接建立成功")

        ws = websocket.WebSocketApp(
            self.__ws_address,
            on_message=on_message,
            on_error=on_error,
            on_close=on_close,
            on_open=on_open,
        )
        # 断开后5秒重连
        ws.run_forever(reconnect=5)

    def __init_events(self):
        """
        初始化事件
        :return:
        """
        self.__events: dict[PostType, dict[str, Callable[[Any], None]]] = {
            PostType.MESSAGE: {
                'on_message_private': self.on_message_private,
                'on_message_group': self.on_message_group
            },
            PostType.MESSAGE_SENT: {
                'on_message_private': self.on_message_private,
                'on_message_group': self.on_message_group
            },
            PostType.NOTICE: {
                'on_notice_friend_recall': self.on_notice_friend_recall,
                'on_notice_group_recall': self.on_notice_group_recall,
                'on_notice_group_increase': self.on_notice_group_increase,
                'on_notice_group_decrease': self.on_notice_group_decrease,
                'on_notice_group_admin': self.on_notice_group_admin,
                'on_notice_group_upload': self.on_notice_group_upload,
                'on_notice_group_ban': self.on_notice_group_ban,
                'on_notice_friend_add': self.on_notice_friend_add,
                'on_notice_notify_poke': self.on_notice_notify_poke,
                'on_notice_notify_lucky_king': self.on_notice_notify_lucky_king,
                'on_notice_notify_honor': self.on_notice_notify_honor,
                'on_notice_notify_title': self.on_notice_notify_title,
                'on_notice_group_card': self.on_notice_group_card,
                'on_notice_offline_file': self.on_notice_offline_file,
                'on_notice_client_status': self.on_notice_client_status,
                'on_notice_essence': self.on_notice_essence,
            },
            PostType.REQUEST: {
                'on_request_friend': self.on_request_friend,
                'on_request_group': self.on_request_group,
            },
            PostType.META_EVENT: {
                'on_meta_event_heartbeat': self.on_meta_event_heartbeat,
                'on_meta_event_lifecycle': self.on_meta_event_lifecycle,
            },
        }

    def __call(self, message_data: dict[str, Any]):
        """
        根据消息类型调用指定的方法
        :param message_data:
        :return:
        """
        pt = message_data.get('post_type', None)
        if pt is None:
            return

        post_type = PostType(pt)
        events = self.__events.get(post_type, None)

        if events is None:
            return

        if post_type == PostType.MESSAGE:
            msg = EventMessage(message_data)
            ev = f'on_{msg.post_type.value}_{msg.message_type.value}'
            fn = events.get(ev, None)
            if fn is None:
                return
            fn(self.__action, msg)

        elif post_type == PostType.MESSAGE_SENT:
            msg = EventMessage(message_data)
            ev = f'on_{msg.post_type.value}_{msg.message_type.value}'
            fn = events.get(ev, None)
            if fn is None:
                return
            fn(self.__action, msg)

        elif post_type == PostType.NOTICE:
            msg = EventNotice(message_data)
            ev = f'on_{msg.post_type.value}_{msg.notice_type.value}'
            if msg.notice_type == PostNoticeType.NOTIFY:
                ev += f'_{msg.sub_type}'
            fn = events.get(ev, None)
            if fn is None:
                return
            fn(self.__action, msg)

        elif post_type == PostType.REQUEST:
            msg = EventRequest(message_data)
            ev = f'on_{msg.post_type.value}_{msg.request_type.value}'
            fn = events.get(ev, None)
            if fn is None:
                return
            fn(self.__action, msg)

        elif post_type == PostType.META_EVENT:
            msg = EventMetaEvent(message_data)
            ev = f'on_{msg.post_type.value}_{msg.meta_event_type.value}'
            fn = events.get(ev, None)
            if fn is None:
                return
            fn(self.__action, msg)


__all__ = ['Bot']
